@@ -7,10 +7,20 @@ import y_py as Y
 
 class YMessageType(IntEnum):
     SYNC = 0
+    AWARENESS = 1
+
+    def raw_str(self) -> str:
+        return self.name
+
+
+class YSyncMessageType(IntEnum):
     SYNC_STEP1 = 0
     SYNC_STEP2 = 1
     SYNC_UPDATE = 2
 
+    def raw_str(self) -> str:
+        return self.name
+
 
 def write_var_uint(num: int) -> bytes:
     res = []
@@ -26,15 +36,15 @@ def create_message(data: bytes, msg_type: int) -> bytes:
 
 
 def create_sync_step1_message(data: bytes) -> bytes:
-    return create_message(data, YMessageType.SYNC_STEP1)
+    return create_message(data, YSyncMessageType.SYNC_STEP1)
 
 
 def create_sync_step2_message(data: bytes) -> bytes:
-    return create_message(data, YMessageType.SYNC_STEP2)
+    return create_message(data, YSyncMessageType.SYNC_STEP2)
 
 
 def create_update_message(data: bytes) -> bytes:
-    return create_message(data, YMessageType.SYNC_UPDATE)
+    return create_message(data, YSyncMessageType.SYNC_UPDATE)
 
 
 def read_message(stream: bytes) -> bytes:
@@ -91,26 +101,33 @@ class Decoder:
 
 
 def put_updates(update_queue: asyncio.Queue, ydoc: Y.YDoc, event: Y.AfterTransactionEvent) -> None:
-    message = create_update_message(event.get_update())
-    update_queue.put_nowait(message)
+    update_queue.put_nowait(event.get_update())
 
 
-async def process_message(message: bytes, ydoc: Y.YDoc, websocket):
-    if message[0] == YMessageType.SYNC:
-        message_type = message[1]
-        msg = message[2:]
-        if message_type == YMessageType.SYNC_STEP1:
-            state = read_message(msg)
-            update = Y.encode_state_as_update(ydoc, state)
-            reply = create_sync_step2_message(update)
-            await websocket.send(reply)
-        elif message_type in (
-            YMessageType.SYNC_STEP2,
-            YMessageType.SYNC_UPDATE,
-        ):
-            update = read_message(msg)
-            Y.apply_update(ydoc, update)
-            return update
+async def process_sync_message(message: bytes, ydoc: Y.YDoc, websocket, log) -> None:
+    message_type = message[0]
+    msg = message[1:]
+    log.debug(
+        "Received %s message from endpoint: %s",
+        YSyncMessageType(message_type).raw_str(),
+        websocket.path,
+    )
+    if message_type == YSyncMessageType.SYNC_STEP1:
+        state = read_message(msg)
+        update = Y.encode_state_as_update(ydoc, state)
+        reply = create_sync_step2_message(update)
+        log.debug(
+            "Sending %s message to endpoint: %s",
+            YSyncMessageType.SYNC_STEP2.raw_str(),
+            websocket.path,
+        )
+        await websocket.send(reply)
+    elif message_type in (
+        YSyncMessageType.SYNC_STEP2,
+        YSyncMessageType.SYNC_UPDATE,
+    ):
+        update = read_message(msg)
+        Y.apply_update(ydoc, update)
 
 
 async def sync(ydoc: Y.YDoc, websocket):
